c*******************************************************************
c*                        REPLICATE                                *
c*                                                                 *
c* Short description:                                              *
c*   Reads from stdin a clon table and a meta-script. It then      *
c*   replaces all ocurrences of certain strings by clon names,     *
c*   nicknames, etc and replicates this result for all clons in    *
c*   the clon table. Writes to stdout.                             *
c*                                                                 *
c* Last update:                                                    *
c*   2003-12-5  Created                                            *
c*                                                                 *
c* Josep Sempau, email: josep.sempau@upc.es                        *
c* Universitat Politecnica de Catalunya, Barcelona, Spain          *
c*******************************************************************
      program main
      implicit none
      integer maxcln,maxlin
      parameter (maxcln=16384,maxlin=128)
      character*256 clon(maxcln),line(maxlin)
      character*32 nick(maxcln),seed1(maxcln),seed2(maxcln),make(maxcln)
      integer nclons,nlines

c     *** Read clon table:
      call clntab(maxcln,clon,nick,seed1,seed2,make,nclons)

c     *** Read meta-script:
      call rdscpt(maxlin,line,nlines)

c     *** Write expanded script:
      call wrscpt(nlines,line,nclons,clon,nick,seed1,seed2,make)
      stop
      end


      subroutine clntab(maxcln,clon,nick,seed1,seed2,make,nclon)
c*******************************************************************
c*    Reads clon table file from stdin                             *
c*                                                                 *
c*    Input:                                                       *
c*      maxcln -> max no. of clons                                 *
c*    Output:                                                      *
c*      clon   -> array of clon names                              *
c*      nick   -> array of clon nicknames                          *
c*      seed1  -> array of 1st seeds                               *
c*      seed2  -> array of 2nd seeds                               *
c*      make   -> make file to compile the job                     *
c*      nclon  -> no. of active clons                              *
c*******************************************************************
      implicit none
      integer maxcln,nclon
      character*(*) clon(maxcln),nick(maxcln)
      character*(*) seed1(maxcln),seed2(maxcln), make(maxcln)

      character*256 buffer
      integer k,j,error

      k = 1
 10   continue
        if (k.gt.maxcln) then
          open(66,file='error.log')
          write(66,*) 'clntab:ERROR: too many clons in clon table.'
          write(*,*) 'cat error.log'
          stop
        endif
        call getlin(buffer,error)
        if (error.ne.0) then
          open(66,file='error.log')
          write(66,*) 'clntab:ERROR: likely, missing EndOfTable.'
          write(*,*) 'cat error.log'
          stop
        endif

c       *** Check EOT condition:
        if (index(buffer,'[EndOfTable]').ne.0) then
          nclon = k-1
          return
        endif

c       *** Parse names and seeds:
        call parse(buffer,clon(k),nick(k),seed1(k),seed2(k), make(k))

c       *** Check consistency:
        do j=1,k-1
          if (nick(k).eq.nick(j)) then
            open(66,file='error.log')
            write(66,*) 'clntab:ERROR: clons with same nickname:'
            write(66,*) nick(k)
            write(*,*) 'cat error.log'
            stop
          endif
          if (seed1(k).eq.seed1(j).and.seed2(k).eq.seed2(j)) then
            open(66,file='error.log')
            write(66,*) 'clntab:ERROR: found clons with same seeds:'
            write(66,*) seed1(k)
            write(66,*) seed2(k)
            write(*,*) 'cat error.log'
            stop
          endif
        enddo
        k = k+1
      goto 10
      end


      subroutine parse(buffer,clon,nick,seed1,seed2, make)
c*******************************************************************
c*    Parses names and seeds from buffer                           *
c*                                                                 *
c*    Input:                                                       *
c*      buffer -> line to parse                                    *
c*    Output:                                                      *
c*      clon   -> clon name                                        *
c*      nick   -> nickname                                         *
c*      seed1, seed2 -> rng seeds                                  *
c*      make   -> make file to compile the job                     *
c*******************************************************************
      implicit none
      character*(*) buffer,clon,nick,seed1,seed2, make

      integer j,k

c     *** Capture clon name:
      call capstr(buffer,clon,j)
      if (j.eq.0) then
        open(66,file='error.log')
        write(66,*) 'parse:ERROR: unable to read clon name from input:'
        write(66,*) buffer
        write(*,*) 'cat error.log'
        stop
      endif

c     *** Capture nickname:
      call capstr(buffer(j+1:),nick,k)
      if (k.eq.0) then
        open(66,file='error.log')
        write(66,*) 'parse:ERROR: unable to read nickname from input:'
        write(66,*) buffer
        write(*,*) 'cat error.log'
        stop
      endif
      j = j+k

c     *** Read seeds:
      call capstr(buffer(j+1:),seed1,k)
      if (k.eq.0) then
        open(66,file='error.log')
        write(66,*) 'parse:ERROR: unable to read seed1 from input:'
        write(66,*) buffer
        write(*,*) 'cat error.log'
        stop
      endif
      j = j+k

      call capstr(buffer(j+1:),seed2,k)
      if (k.eq.0) then
        open(66,file='error.log')
        write(66,*) 'parse:ERROR: unable to read seed2 from input:'
        write(66,*) buffer
        write(*,*) 'cat error.log'
        stop
      endif
      j = j+k

c     *** Capture make file name:
      call capstr(buffer(j+1:),make,k)
      if (k.eq.0) then
        open(66,file='error.log')
        write(66,*)
     &     'parse:ERROR: unable to read the make file name from input:'
        write(66,*) buffer
        write(*,*) 'cat error.log'
        stop
      endif
      end


      subroutine capstr(string,substr,pos)
c*******************************************************************
c*    Captures the substring of string that starts after the 1st   *
c*    non-blank character and ends when the next blank is found.   *
c*                                                                 *
c*    Input:                                                       *
c*      string -> character string                                 *
c*    Output:                                                      *
c*      substr -> substring                                        *
c*      pos -> last position read from string into substr          *
c*******************************************************************
      implicit none
      character*(*) string,substr
      integer pos

      character nextch
      integer i,j,k

      pos = 0
      substr = ' '
      do j=1,len(string)
        if (string(j:j).ne.' ') goto 10
      enddo
      return
 10   continue

      do k=1,len(string)-j+1
        i = j+k-1
        nextch = string(i:i)
        if (nextch.eq.' ') return
        substr(k:k) = nextch
        pos = i
      enddo
      end


      integer function str1st(string)
c*******************************************************************
c*    Returns position in string of first non-blank character.     *
c*    If string is all blanks, it returns zero.                    *
c*                                                                 *
c*    Input:                                                       *
c*      string -> character string                                 *
c*******************************************************************
      implicit none
      character*(*) string

      do str1st=1,len(string)
        if (string(str1st:str1st).ne.' ') return
      enddo
      str1st = 0
      end


      subroutine rdscpt(maxlin,line,nlines)
c*******************************************************************
c*    Reads meta-script                                            *
c*                                                                 *
c*    Input:                                                       *
c*      maxlin -> max number of lines                              *
c*    Output:                                                      *
c*      line -> array of text lines of meta-script                 *
c*      nlines -> actual no. of lines in meta-script               *
c*******************************************************************
      implicit none
      integer maxlin,nlines
      character*(*) line(maxlin)

      character*256 buffer
      integer error

      nlines = 0

c     *** Locate starting mark:
 10   continue
        call getlin(buffer,error)
        if (error.ne.0) then
          open(66,file='error.log')
          write(66,*) 'rdscpt:ERROR: [REPEAT] not found.'
          write(*,*) 'cat error.log'
          stop
        endif
      if (buffer(:8).ne.'[REPEAT]') goto 10

c     *** Load meta-script:
 20   continue
        call getlin(buffer,error)
        if (error.ne.0) return
        nlines = nlines+1
        if (nlines.eq.maxlin) then
          open(66,file='error.log')
          write(66,*) 'rdscpt:ERROR: too many lines in meta-script;'
          write(66,*) '              enlarge MAXLIN.'
          write(*,*) 'cat error.log'
          stop
        endif
        line(nlines) = buffer
      goto 20
      end


      subroutine wrscpt(nlines,line,nclons,clon,nick,seed1,seed2,make)
c*******************************************************************
c*    Writes expanded script combining meta-script and clon table. *
c*                                                                 *
c*    Input:                                                       *
c*      nlines -> no. of lines in meta-script                      *
c*      line   -> array of lines of meta-script                    *
c*      nclons -> no. of clons                                     *
c*      clon   -> array of clon names                              *
c*      nick   -> array of nicknames                               *
c*      seed1, seed2 -> arrays of seeds                            *
c*    Output:                                                      *
c*      text is written to stdout                                  *
c*******************************************************************
      implicit none
      integer nlines,nclons
      character*(*) line(nlines),clon(nclons),nick(nclons)
      character*(*) seed1(nclons),seed2(nclons), make(nclons)

      integer i,j,strlen,k
      character*256 buffer,string

      do i=1,nclons
        write(*,*) ' '
        do j=1,nlines
          string = line(j)
c         *** Replace '%name':
 10       continue
            k = index(string,'%name')
            if (k.eq.0) goto 20
            if (k.gt.1) then
              buffer =
     &          string(:k-1)//clon(i)(:strlen(clon(i)))//string(k+5:)
            else
              buffer = clon(i)(:strlen(clon(i)))//string(k+5:)
            endif
            string = buffer
          goto 10
 20       continue
c         *** Replace '%nick':
 30       continue
            k = index(string,'%nick')
            if (k.eq.0) goto 40
            if (k.gt.1) then
              buffer =
     &          string(:k-1)//nick(i)(:strlen(nick(i)))//string(k+5:)
            else
              buffer = nick(i)(:strlen(nick(i)))//string(k+5:)
            endif
            string = buffer
          goto 30
 40       continue
c         *** Replace '%seed1':
 50       continue
            k = index(string,'%seed1')
            if (k.eq.0) goto 60
            if (k.gt.1) then
              buffer =
     &        string(:k-1)//seed1(i)(:strlen(seed1(i)))//string(k+6:)
            else
              buffer = seed1(i)(:strlen(seed1(i)))//string(k+6:)
            endif
            string = buffer
          goto 50
 60       continue
c         *** Replace '%seed2':
 70       continue
            k = index(string,'%seed2')
            if (k.eq.0) goto 80
            if (k.gt.1) then
              buffer =
     &        string(:k-1)//seed2(i)(:strlen(seed2(i)))//string(k+6:)
            else
              buffer = seed2(i)(:strlen(seed2(i)))//string(k+6:)
            endif
            string = buffer
          goto 70
 80       continue

c         *** Replace '%makefile':
 90       continue
            k = index(string,'%makefile')
            if (k.eq.0) goto 100
            if (k.gt.1) then
              buffer =
     &          string(:k-1)//make(i)(:strlen(make(i)))//string(k+9:)
            else
              buffer = make(i)(:strlen(make(i)))//string(k+9:)
            endif
            string = buffer

          goto 90
 100      continue


c         *** Write line after all replacements:
          write(*,*) string(:strlen(string))
        enddo
      enddo
      write(*,*) ' '
      end


      integer function strlen(string)
c*******************************************************************
c*    Returns length of string excluding trailing blanks           *
c*                                                                 *
c*    Input:                                                       *
c*      string -> character string                                 *
c*******************************************************************
      implicit none
      character*(*) string

      do strlen=len(string),1,-1
        if (string(strlen:strlen).ne.' ') return
      enddo
      strlen = 0
      end


      subroutine getlin(buffer,error)
c*******************************************************************
c*    Reads a non-comment line from stdin                          *
c*                                                                 *
c*    Output:                                                      *
c*      buffer -> line just read                                   *
c*      error -> not 0 if an error has ocurred                     *
c*******************************************************************
      implicit none
      character*(*) buffer
      integer error,str1st

 10   continue
      read(*,'(a)',iostat=error) buffer
      if (error.ne.0) return
      if (buffer(1:1).eq.'#') goto 10
      if (str1st(buffer).eq.0) goto 10
      end


c >>>> EOF >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

